@@ -35,7 +35,7 @@ usage()
 {
     TFPRINTF(stderr,
         _TSK_T
-        ("usage: %s [-aAelvV] [-f fstype] [-i imgtype] [-o imgoffset] image [images] [start-stop]\n"),
+        ("usage: %s [-aAelvV] [-f fstype] [-i imgtype] [-b dev_sector_size] [-o imgoffset] image [images] [start-stop]\n"),
         progname);
     tsk_fprintf(stderr, "\t-e: every block\n");
     tsk_fprintf(stderr,
@@ -46,6 +46,8 @@ usage()
         "\t-f fstype: File system type (use '-f list' for supported types)\n");
     tsk_fprintf(stderr,
         "\t-i imgtype: The format of the image file (use '-i list' for supported types)\n");
+    tsk_fprintf(stderr,
+        "\t-b dev_sector_size: The size (in bytes) of the device sectors\n");
     tsk_fprintf(stderr,
         "\t-o imgoffset: The offset of the file system in the image (in sectors)\n");
     tsk_fprintf(stderr,
@@ -64,12 +66,12 @@ usage()
 /* main - open file system, list block info */
 
 int
-main(int argc, char ** argv1)
+main(int argc, char **argv1)
 {
     TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_IMG_INFO *img;
 
-    TSK_OFF_T imgoff = 0;
+    TSK_OFF_T imgaddr = 0;
     TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
     TSK_FS_INFO *fs;
 
@@ -82,22 +84,23 @@ main(int argc, char ** argv1)
 
     char lclflags = TSK_FS_BLKLS_CAT, set_bounds = 1;
     TSK_TCHAR **argv;
-    
+    unsigned int ssize = 0;
+
 #ifdef TSK_WIN32
     // On Windows, get the wide arguments (mingw doesn't support wmain)
     argv = CommandLineToArgvW(GetCommandLineW(), &argc);
-    if( argv == NULL) {    
+    if (argv == NULL) {
         fprintf(stderr, "Error getting wide arguments\n");
         exit(1);
     }
 #else
-    argv = (TSK_TCHAR **)argv1;
+    argv = (TSK_TCHAR **) argv1;
 #endif
-    
+
     progname = argv[0];
     setlocale(LC_ALL, "");
 
-    while ((ch = GETOPT(argc, argv, _TSK_T("aAef:i:lo:svV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("aAb:ef:i:lo:svV"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
@@ -106,9 +109,21 @@ main(int argc, char ** argv1)
             usage();
         case _TSK_T('a'):
             flags |= TSK_FS_BLOCK_FLAG_ALLOC;
+            flags &= ~TSK_FS_BLOCK_FLAG_UNALLOC;
             break;
         case _TSK_T('A'):
             flags |= TSK_FS_BLOCK_FLAG_UNALLOC;
+            flags &= ~TSK_FS_BLOCK_FLAG_ALLOC;
+            break;
+        case _TSK_T('b'):
+            ssize = (unsigned int) TSTRTOUL(OPTARG, &cp, 0);
+            if (*cp || *cp == *OPTARG || ssize < 1) {
+                TFPRINTF(stderr,
+                    _TSK_T
+                    ("invalid argument: sector size must be positive: %s\n"),
+                    OPTARG);
+                usage();
+            }
             break;
         case _TSK_T('e'):
             flags |= (TSK_FS_BLOCK_FLAG_ALLOC | TSK_FS_BLOCK_FLAG_UNALLOC);
@@ -141,7 +156,7 @@ main(int argc, char ** argv1)
             lclflags = TSK_FS_BLKLS_LIST;
             break;
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
+            if ((imgaddr = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
@@ -174,22 +189,20 @@ main(int argc, char ** argv1)
         }
 
         /* There should be no other arguments */
-        img =
-            tsk_img_open(argc - OPTIND,
-            (const TSK_TCHAR **) &argv[OPTIND], imgtype);
+        img = tsk_img_open(argc - OPTIND, &argv[OPTIND], imgtype, ssize);
 
         if (img == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
-        if (imgoff >= img->size) {
+        if ((imgaddr * img->sector_size) >= img->size) {
             tsk_fprintf(stderr,
                 "Sector offset supplied is larger than disk image (maximum: %"
-                PRIu64 ")\n", img->size / 512);
+                PRIu64 ")\n", img->size / img->sector_size);
             exit(1);
         }
 
-        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgaddr * img->sector_size, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
                 tsk_fs_type_print(stderr);
@@ -203,16 +216,15 @@ main(int argc, char ** argv1)
         if ((dash = TSTRCHR(argv[argc - 1], _TSK_T('-'))) == NULL) {
             /* No dash in arg - therefore it is an image file name */
             if ((img =
-                    tsk_img_open(argc - OPTIND,
-                        (const TSK_TCHAR **) &argv[OPTIND],
-                        imgtype)) == NULL) {
+                    tsk_img_open(argc - OPTIND, &argv[OPTIND],
+                        imgtype, ssize)) == NULL) {
                 tsk_error_print(stderr);
                 exit(1);
             }
-            if (imgoff >= img->size) {
+            if ((imgaddr * img->sector_size) >= img->size) {
                 tsk_fprintf(stderr,
                     "Sector offset supplied is larger than disk image (maximum: %"
-                    PRIu64 ")\n", img->size / 512);
+                    PRIu64 ")\n", img->size / img->sector_size);
                 exit(1);
             }
 
@@ -227,16 +239,15 @@ main(int argc, char ** argv1)
                 /* Not a number - consider it a file name */
                 *dash = _TSK_T('-');
                 if ((img =
-                        tsk_img_open(argc - OPTIND,
-                            (const TSK_TCHAR **) &argv[OPTIND],
-                            imgtype)) == NULL) {
+                        tsk_img_open(argc - OPTIND, &argv[OPTIND],
+                            imgtype, ssize)) == NULL) {
                     tsk_error_print(stderr);
                     exit(1);
                 }
-                if (imgoff >= img->size) {
+                if ((imgaddr * img->sector_size) >= img->size) {
                     tsk_fprintf(stderr,
                         "Sector offset supplied is larger than disk image (maximum: %"
-                        PRIu64 ")\n", img->size / 512);
+                        PRIu64 ")\n", img->size / img->sector_size);
                     exit(1);
                 }
 
@@ -251,16 +262,15 @@ main(int argc, char ** argv1)
                     dash--;
                     *dash = _TSK_T('-');
                     if ((img =
-                            tsk_img_open(argc - OPTIND,
-                                (const TSK_TCHAR **) &argv[OPTIND],
-                                imgtype)) == NULL) {
+                            tsk_img_open(argc - OPTIND, &argv[OPTIND],
+                                imgtype, ssize)) == NULL) {
                         tsk_error_print(stderr);
                         exit(1);
                     }
-                    if (imgoff >= img->size) {
+                    if ((imgaddr * img->sector_size) >= img->size) {
                         tsk_fprintf(stderr,
                             "Sector offset supplied is larger than disk image (maximum: %"
-                            PRIu64 ")\n", img->size / 512);
+                            PRIu64 ")\n", img->size / img->sector_size);
                         exit(1);
                     }
 
@@ -271,23 +281,22 @@ main(int argc, char ** argv1)
                     set_bounds = 0;
                     /* It was a block range, so do not include it in the open */
                     if ((img =
-                            tsk_img_open(argc - OPTIND - 1,
-                                (const TSK_TCHAR **) &argv[OPTIND],
-                                imgtype)) == NULL) {
+                            tsk_img_open(argc - OPTIND - 1, &argv[OPTIND],
+                                imgtype, ssize)) == NULL) {
                         tsk_error_print(stderr);
                         exit(1);
                     }
-                    if (imgoff >= img->size) {
+                    if ((imgaddr * img->sector_size) >= img->size) {
                         tsk_fprintf(stderr,
                             "Sector offset supplied is larger than disk image (maximum: %"
-                            PRIu64 ")\n", img->size / 512);
+                            PRIu64 ")\n", img->size / img->sector_size);
                         exit(1);
                     }
                 }
             }
         }
 
-        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgaddr * img->sector_size, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
                 tsk_fs_type_print(stderr);
