@@ -30,7 +30,7 @@ usage()
 {
     TFPRINTF(stderr,
         _TSK_T
-        ("usage: %s [-adDFlpruvV] [-f fstype] [-i imgtype] [-m dir/] [-o imgoffset] [-z ZONE] [-s seconds] image [images] [inode]\n"),
+        ("usage: %s [-adDFlpruvV] [-f fstype] [-i imgtype] [-b dev_sector_size] [-m dir/] [-o imgoffset] [-z ZONE] [-s seconds] image [images] [inode]\n"),
         progname);
     tsk_fprintf(stderr,
         "\tIf [inode] is not given, the root directory is used\n");
@@ -41,6 +41,8 @@ usage()
     tsk_fprintf(stderr, "\t-l: Display long version (like ls -l)\n");
     tsk_fprintf(stderr,
         "\t-i imgtype: Format of image file (use '-i list' for supported types)\n");
+    tsk_fprintf(stderr,
+        "\t-b dev_sector_size: The size (in bytes) of the device sectors\n");
     tsk_fprintf(stderr,
         "\t-f fstype: File system type (use '-f list' for supported types)\n");
     tsk_fprintf(stderr,
@@ -63,12 +65,12 @@ usage()
 }
 
 int
-main(int argc, char ** argv1)
+main(int argc, char **argv1)
 {
     TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
     TSK_IMG_INFO *img;
 
-    TSK_OFF_T imgoff = 0;
+    TSK_OFF_T imgaddr = 0;
     TSK_FS_TYPE_ENUM fstype = TSK_FS_TYPE_DETECT;
     TSK_FS_INFO *fs;
 
@@ -80,24 +82,27 @@ main(int argc, char ** argv1)
     int32_t sec_skew = 0;
     static TSK_TCHAR *macpre = NULL;
     TSK_TCHAR **argv;
-    
+    unsigned int ssize = 0;
+    TSK_TCHAR *cp;
+
 #ifdef TSK_WIN32
     // On Windows, get the wide arguments (mingw doesn't support wmain)
     argv = CommandLineToArgvW(GetCommandLineW(), &argc);
-    if( argv == NULL) {    
+    if (argv == NULL) {
         fprintf(stderr, "Error getting wide arguments\n");
         exit(1);
     }
 #else
-    argv = (TSK_TCHAR **)argv1;
+    argv = (TSK_TCHAR **) argv1;
 #endif
-    
+
     progname = argv[0];
     setlocale(LC_ALL, "");
 
     fls_flags = TSK_FS_FLS_DIR | TSK_FS_FLS_FILE;
 
-    while ((ch = GETOPT(argc, argv, _TSK_T("adDf:Fi:m:lo:prs:uvVz:"))) > 0) {
+    while ((ch =
+            GETOPT(argc, argv, _TSK_T("ab:dDf:Fi:m:lo:prs:uvVz:"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
@@ -107,6 +112,16 @@ main(int argc, char ** argv1)
         case _TSK_T('a'):
             fls_flags |= TSK_FS_FLS_DOT;
             break;
+        case _TSK_T('b'):
+            ssize = (unsigned int) TSTRTOUL(OPTARG, &cp, 0);
+            if (*cp || *cp == *OPTARG || ssize < 1) {
+                TFPRINTF(stderr,
+                    _TSK_T
+                    ("invalid argument: sector size must be positive: %s\n"),
+                    OPTARG);
+                usage();
+            }
+            break;
         case _TSK_T('d'):
             name_flags &= ~TSK_FS_NAME_FLAG_ALLOC;
             break;
@@ -150,7 +165,7 @@ main(int argc, char ** argv1)
             macpre = OPTARG;
             break;
         case _TSK_T('o'):
-            if ((imgoff = tsk_parse_offset(OPTARG)) == -1) {
+            if ((imgaddr = tsk_parse_offset(OPTARG)) == -1) {
                 tsk_error_print(stderr);
                 exit(1);
             }
@@ -234,19 +249,18 @@ main(int argc, char ** argv1)
     if (tsk_fs_parse_inum(argv[argc - 1], &inode, NULL, NULL, NULL, NULL)) {
         /* Not an inode at the end */
         if ((img =
-                tsk_img_open(argc - OPTIND,
-                    (const TSK_TCHAR **) &argv[OPTIND],
-                    imgtype)) == NULL) {
+                tsk_img_open(argc - OPTIND, &argv[OPTIND],
+                    imgtype, ssize)) == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
-        if (imgoff >= img->size) {
+        if ((imgaddr * img->sector_size) >= img->size) {
             tsk_fprintf(stderr,
                 "Sector offset supplied is larger than disk image (maximum: %"
-                PRIu64 ")\n", img->size / 512);
+                PRIu64 ")\n", img->size / img->sector_size);
             exit(1);
         }
-        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgaddr * img->sector_size, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
                 tsk_fs_type_print(stderr);
@@ -264,21 +278,20 @@ main(int argc, char ** argv1)
         }
 
         if ((img =
-                tsk_img_open(argc - OPTIND - 1,
-                    (const TSK_TCHAR **) &argv[OPTIND],
-                    imgtype)) == NULL) {
+                tsk_img_open(argc - OPTIND - 1, &argv[OPTIND],
+                    imgtype, ssize)) == NULL) {
             tsk_error_print(stderr);
             exit(1);
         }
-        if (imgoff >= img->size) {
+        if ((imgaddr * img->sector_size) >= img->size) {
             tsk_fprintf(stderr,
                 "Sector offset supplied is larger than disk image (maximum: %"
-                PRIu64 ")\n", img->size / 512);
+                PRIu64 ")\n", img->size / img->sector_size);
             exit(1);
         }
 
 
-        if ((fs = tsk_fs_open_img(img, imgoff, fstype)) == NULL) {
+        if ((fs = tsk_fs_open_img(img, imgaddr * img->sector_size, fstype)) == NULL) {
             tsk_error_print(stderr);
             if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
                 tsk_fs_type_print(stderr);
