@@ -19,10 +19,14 @@ static TSK_TCHAR *progname;
 static void
 usage()
 {
-    TFPRINTF(stderr, _TSK_T("usage: %s [-vV] [-i imgtype] image\n"),
+    TFPRINTF(stderr,
+        _TSK_T
+        ("usage: %s [-vV] [-i imgtype] [-b dev_sector_size] image\n"),
         progname);
     tsk_fprintf(stderr,
         "\t-i imgtype: The format of the image file (use 'i list' for supported types)\n");
+    tsk_fprintf(stderr,
+        "\t-b dev_sector_size: The size (in bytes) of the device sectors\n");
     tsk_fprintf(stderr, "\t-v: verbose output to stderr\n");
     tsk_fprintf(stderr, "\t-V: Print version\n");
 
@@ -31,7 +35,7 @@ usage()
 
 
 int
-main(int argc, char ** argv1)
+main(int argc, char **argv1)
 {
     TSK_IMG_INFO *img;
     TSK_IMG_TYPE_ENUM imgtype = TSK_IMG_TYPE_DETECT;
@@ -39,28 +43,39 @@ main(int argc, char ** argv1)
     TSK_OFF_T done;
     ssize_t cnt;
     TSK_TCHAR **argv;
-    
+    unsigned int ssize = 0;
+    TSK_TCHAR *cp;
+
 #ifdef TSK_WIN32
     // On Windows, get the wide arguments (mingw doesn't support wmain)
     argv = CommandLineToArgvW(GetCommandLineW(), &argc);
-    if( argv == NULL) {    
+    if (argv == NULL) {
         fprintf(stderr, "Error getting wide arguments\n");
         exit(1);
     }
 #else
-    argv = (TSK_TCHAR **)argv1;
+    argv = (TSK_TCHAR **) argv1;
 #endif
 
     progname = argv[0];
 
-    while ((ch = GETOPT(argc, argv, _TSK_T("i:vV"))) > 0) {
+    while ((ch = GETOPT(argc, argv, _TSK_T("b:i:vV"))) > 0) {
         switch (ch) {
         case _TSK_T('?'):
         default:
             TFPRINTF(stderr, _TSK_T("Invalid argument: %s\n"),
                 argv[OPTIND]);
             usage();
-
+        case _TSK_T('b'):
+            ssize = (unsigned int) TSTRTOUL(OPTARG, &cp, 0);
+            if (*cp || *cp == *OPTARG || ssize < 1) {
+                TFPRINTF(stderr,
+                    _TSK_T
+                    ("invalid argument: sector size must be positive: %s\n"),
+                    OPTARG);
+                usage();
+            }
+            break;
         case _TSK_T('i'):
             if (TSTRCMP(OPTARG, _TSK_T("list")) == 0) {
                 tsk_img_type_print(stderr);
@@ -91,17 +106,16 @@ main(int argc, char ** argv1)
     }
 
     if ((img =
-            tsk_img_open(argc - OPTIND,
-                (const TSK_TCHAR **) &argv[OPTIND], imgtype)) == NULL) {
+            tsk_img_open(argc - OPTIND, &argv[OPTIND], imgtype,
+                ssize)) == NULL) {
         tsk_error_print(stderr);
         exit(1);
     }
 
 #ifdef TSK_WIN32
     if (-1 == _setmode(_fileno(stdout), _O_BINARY)) {
-        fprintf(stderr, 
-            "error setting stdout to binary: %s",
-            strerror(errno));
+        fprintf(stderr,
+            "error setting stdout to binary: %s", strerror(errno));
         exit(1);
     }
 #endif
@@ -110,7 +124,7 @@ main(int argc, char ** argv1)
         char buf[16 * 1024];
         size_t len;
 
-        if (done + (TSK_OFF_T)sizeof(buf) > img->size) {
+        if (done + (TSK_OFF_T) sizeof(buf) > img->size) {
             len = (size_t) (img->size - done);
         }
         else {
@@ -118,7 +132,7 @@ main(int argc, char ** argv1)
         }
 
         cnt = tsk_img_read(img, done, buf, len);
-        if (cnt != (ssize_t)len) {
+        if (cnt != (ssize_t) len) {
             if (cnt >= 0) {
                 tsk_fprintf(stderr,
                     "img_cat: Error reading image file at offset: %"
@@ -133,7 +147,7 @@ main(int argc, char ** argv1)
         }
 
         if (fwrite(buf, cnt, 1, stdout) != 1) {
-            fprintf(stderr, 
+            fprintf(stderr,
                 "img_cat: Error writing to stdout:  %s", strerror(errno));
             tsk_img_close(img);
             exit(1);
