@@ -235,6 +235,109 @@ tsk_UTF16toUTF8(TSK_ENDIAN_ENUM endian, const UTF16 ** sourceStart,
     return result;
 }
 
+
+/** 
+* \ingroup baselib
+* Convert a UTF-16 string in local endian ordering to UTF-8.  
+* @param sourceStart Pointer to pointer to start of UTF-16 string.  Will be updated to last char proccessed.
+* @param sourceEnd Pointer to one entry past end of UTF-16 string
+* @param targetStart Pointer to pointer to place where UTF-8 string should be written.  Will be updated to next place to write to. 
+* @param targetEnd Pointer to end of UTF-8 buffer
+* @param flags Flags used during conversion 
+* @returns error code
+*/
+TSKConversionResult
+tsk_UTF16toUTF8_lclorder(const UTF16 ** sourceStart,
+    const UTF16 * sourceEnd, UTF8 ** targetStart,
+    UTF8 * targetEnd, TSKConversionFlags flags)
+{
+    TSKConversionResult result = TSKconversionOK;
+    const UTF16 *source = *sourceStart;
+    UTF8 *target = *targetStart;
+    while (source < sourceEnd) {
+        UTF32 ch;
+        unsigned short bytesToWrite = 0;
+        const UTF32 byteMask = 0xBF;
+        const UTF32 byteMark = 0x80;
+        const UTF16 *oldSource = source;        /* In case we have to back up because of target overflow. */
+        ch = *source++;
+
+        /* If we have a surrogate pair, convert to UTF32 first. */
+        if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_HIGH_END) {
+            /* If the 16 bits following the high surrogate are in the source buffer... */
+            if (source < sourceEnd) {
+                UTF32 ch2 = *source;
+                /* If it's a low surrogate, convert to UTF32. */
+                if (ch2 >= UNI_SUR_LOW_START && ch2 <= UNI_SUR_LOW_END) {
+                    ch = ((ch - UNI_SUR_HIGH_START) << halfShift)
+                        + (ch2 - UNI_SUR_LOW_START) + halfBase;
+                    ++source;
+                }
+                else if (flags == TSKstrictConversion) {        /* it's an unpaired high surrogate */
+                    --source;   /* return to the illegal value itself */
+                    result = TSKsourceIllegal;
+                    break;
+                }
+            }
+            else {              /* We don't have the 16 bits following the high surrogate. */
+                --source;       /* return to the high surrogate */
+                result = TSKsourceExhausted;
+                break;
+            }
+        }
+        else if (flags == TSKstrictConversion) {
+            /* UTF-16 surrogate values are illegal in UTF-32 */
+            if (ch >= UNI_SUR_LOW_START && ch <= UNI_SUR_LOW_END) {
+                --source;       /* return to the illegal value itself */
+                result = TSKsourceIllegal;
+                break;
+            }
+        }
+/* Figure out how many bytes the result will require */
+        if (ch < (UTF32) 0x80) {
+            bytesToWrite = 1;
+        }
+        else if (ch < (UTF32) 0x800) {
+            bytesToWrite = 2;
+        }
+        else if (ch < (UTF32) 0x10000) {
+            bytesToWrite = 3;
+        }
+        else if (ch < (UTF32) 0x110000) {
+            bytesToWrite = 4;
+        }
+        else {
+            bytesToWrite = 3;
+            ch = TSK_UNI_REPLACEMENT_CHAR;
+        }
+
+        target += bytesToWrite;
+        if (target > targetEnd) {
+            source = oldSource; /* Back up source pointer! */
+            target -= bytesToWrite;
+            result = TSKtargetExhausted;
+            break;
+        }
+        switch (bytesToWrite) { /* note: everything falls through. */
+        case 4:
+            *--target = (UTF8) ((ch | byteMark) & byteMask);
+            ch >>= 6;
+        case 3:
+            *--target = (UTF8) ((ch | byteMark) & byteMask);
+            ch >>= 6;
+        case 2:
+            *--target = (UTF8) ((ch | byteMark) & byteMask);
+            ch >>= 6;
+        case 1:
+            *--target = (UTF8) (ch | firstByteMark[bytesToWrite]);
+        }
+        target += bytesToWrite;
+    }
+    *sourceStart = source;
+    *targetStart = target;
+    return result;
+}
+
 /* --------------------------------------------------------------------- */
 
 /*
