@@ -708,15 +708,15 @@ fatfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     // clean up any error messages that are lying around
     tsk_error_reset();
 
-    if ((data_buf = (char *) tsk_malloc(fatfs->ssize)) == NULL) {
+    if ((data_buf = (char *) tsk_malloc(fs->block_size)) == NULL) {
         return 1;
     }
 
 
     /* Read the root directory sector so that we can get the volume
      * label from it */
-    cnt = tsk_fs_read_block(fs, fatfs->rootsect, data_buf, fatfs->ssize);
-    if (cnt != fatfs->ssize) {
+    cnt = tsk_fs_read_block(fs, fatfs->rootsect, data_buf, fs->block_size);
+    if (cnt != fs->block_size) {
         if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
@@ -778,7 +778,7 @@ fatfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
                 "Volume Label (Root Directory): %c%c%c%c%c%c%c%c%c%c%c\n",
                 de->name[0], de->name[1], de->name[2], de->name[3],
                 de->name[4], de->name[5], de->name[6], de->name[7],
-                de->name[8], de->name[9], de->name[10]);
+                de->ext[0], de->ext[1], de->ext[2]);
         }
         else {
             tsk_fprintf(hFile, "Volume Label (Root Directory):\n");
@@ -818,7 +818,7 @@ fatfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
                 "Volume Label (Root Directory): %c%c%c%c%c%c%c%c%c%c%c\n",
                 de->name[0], de->name[1], de->name[2], de->name[3],
                 de->name[4], de->name[5], de->name[6], de->name[7],
-                de->name[8], de->name[9], de->name[10]);
+                de->ext[0], de->ext[1], de->ext[2]);
         }
         else {
             tsk_fprintf(hFile, "Volume Label (Root Directory):\n");
@@ -1350,7 +1350,7 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         if (i == 0)
             sb_off = 0;
         else
-            sb_off = 6 * 512;   // the backup is located in sector 6
+            sb_off = 6 * img_info->sector_size; // the backup is located in sector 6
 
         cnt = tsk_fs_read(fs, sb_off, (char *) fatsb, len);
         if (cnt != len) {
@@ -1389,7 +1389,7 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         }
     }
 
-    fs->dev_bsize = FATFS_DEV_BSIZE;
+    fs->dev_bsize = img_info->sector_size;
 
     /* Calculate block sizes and layout info */
     // sector size
@@ -1681,6 +1681,7 @@ fatfs_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
     fs->load_attrs = fatfs_make_data_run;
 
     fs->dir_open_meta = fatfs_dir_open_meta;
+    fs->name_cmp = fatfs_name_cmp;
 
     fs->fsstat = fatfs_fsstat;
     fs->fscheck = fatfs_fscheck;
