@@ -1050,7 +1050,8 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
         size_t len_remain;      // length remaining to copy
         size_t len_toread;      // length total to copy
 
-        if (a_offset >= a_fs_attr->nrd.allocsize) {
+        if (((a_flags & TSK_FS_FILE_READ_FLAG_SLACK) && (a_offset >= a_fs_attr->nrd.allocsize)) ||
+            (!(a_flags & TSK_FS_FILE_READ_FLAG_SLACK) && (a_offset >= a_fs_attr->size))) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ_OFF;
             snprintf(tsk_errstr, TSK_ERRSTR_L,
@@ -1094,15 +1095,18 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                 continue;
 
             // block offset into this run
-            blkoffset_inrun = blkoffset_toread - data_run_cur->offset;
+            if (data_run_cur->offset < blkoffset_toread)
+                blkoffset_inrun = blkoffset_toread - data_run_cur->offset;
+            else
+                blkoffset_inrun = 0;
 
             // see if we need to read the rest of this run and into the next or if it is all here
             len_inrun = len_remain;
-            if ((data_run_cur->len - blkoffset_inrun) * fs->block_size <
+            if ((data_run_cur->len - blkoffset_inrun) * fs->block_size - byteoffset_toread <
                 len_remain)
                 len_inrun =
                     (size_t) ((data_run_cur->len -
-                        blkoffset_inrun) * fs->block_size);
+                        blkoffset_inrun) * fs->block_size - byteoffset_toread) ;
 
             /* sparse files/runs just get 0s */
             if (data_run_cur->flags & TSK_FS_ATTR_RUN_FLAG_SPARSE) {
@@ -1168,13 +1172,13 @@ tsk_fs_attr_read(const TSK_FS_ATTR * a_fs_attr, TSK_OFF_T a_offset,
                             byteoffset_toread + len_inrun) >
                         a_fs_attr->nrd.initsize)
                     && ((a_flags & TSK_FS_FILE_READ_FLAG_SLACK) == 0)) {
-                    size_t off =
-                        (size_t) ((data_run_cur->offset +
-                            blkoffset_inrun) * fs->block_size +
-                        byteoffset_toread + len_inrun -
-                        a_fs_attr->nrd.initsize);
-                    memset(&a_buf[len_toread - len_remain + off], 0,
-                        len_inrun - off);
+                    
+                    size_t uninit_off = a_fs_attr->nrd.initsize - 
+                        ((data_run_cur->offset + blkoffset_inrun) * fs->block_size +
+                        byteoffset_toread);
+
+                    memset(&a_buf[len_toread - len_remain + uninit_off], 0,
+                        len_inrun - uninit_off);
                 }
 
             }
