@@ -68,7 +68,7 @@
 
 
 /* free all memory used by inode linked list */
-void
+static void
 iso9660_inode_list_free(TSK_FS_INFO * fs)
 {
     ISO_INFO *iso = (ISO_INFO *) fs;
@@ -93,7 +93,7 @@ iso9660_inode_list_free(TSK_FS_INFO * fs)
  * @param hFile File handle to print details to  (or NULL for no printing)
  * @returns NULL on error
  */
-rockridge_ext *
+static rockridge_ext *
 parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
 {
     rockridge_ext *rr;
@@ -341,7 +341,30 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
     return rr;
 }
 
-/* get directory entries from current directory and add them to the inode list.
+
+///////////////////////////////////////////////////////////////////////////
+// The following functions are responsible for loading all of the file metadata into memory.
+// The process is that the Path table is processed first.  It contains an entry for each
+// directory.  That info is then used to locate the directory contents and those contents
+// are then processed. 
+//
+// Files do not have a corresponding metadata entry, so we assign them based
+// on the order that they are loaded. 
+///////////////////////////////////////////////////////////////////////////
+
+
+/* XXX Instead of loading all of the file metadata, we could instead save a mapping
+ * between inode number and the byte offset of the metadata (and any other data
+ * needed for fast lookups).
+ */
+
+/** \internal
+ * Process the contents of a directory and load the
+ * information about files in that directory into ISO_INFO.  This is called
+ * by the methods that process the path table (which contains pointers to the 
+ * various directories).  The results in ISO_INFO are used to identify the
+ * inode address of files found from dent_walk and for file lookups. 
+ *
  * Type: ISO9660_TYPE_PVD for primary volume descriptor, ISO9660_TYPE_SVD for
  * supplementary volume descriptor (do Joliet utf-8 conversion).
  *
@@ -349,14 +372,13 @@ parse_susp(TSK_FS_INFO * fs, char *buf, int count, FILE * hFile)
  * @param a_offs Byte offset of directory start
  * @param count previous file count
  * @param ctype Character set used for the names
- * @param a_fn Name of the directory  (in UTF-8)
+ * @param a_fn Name of the directory to use for the "." entry (in UTF-8)
  *
  * @returns total number of files or -1 on error
  */
-
-int
+static int
 iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
-    int ctype, char *a_fn)
+    int ctype, const char *a_fn, uint8_t is_first)
 {
     ISO_INFO *iso = (ISO_INFO *) fs;
     int s_cnt = 1;              // count of sectors needed for dir
@@ -397,7 +419,15 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                 b_offs += 2;
                 continue;
             }
-            b_offs += sizeof(iso9660_dentry);
+
+            /* when processing the other volume descriptor directories, we ignore the 
+             * directories because we have no way of detecting if it is a duplicate of
+             * a directory from the other volume descriptor (they use different blocks).
+             * We will see the contents of this directory from the path table anyway. */
+            if ((dentry->flags & ISO9660_FLAG_DIR) && (is_first == 0)) {
+                b_offs += dentry->entry_len;
+                continue;
+            }
 
             // allocate a node for this entry
             in_node = (iso9660_inode_node *)
@@ -407,7 +437,7 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
             }
 
             // the first entry should have no name and is for the current directory
-            if ((i == 0) && (b_offs == sizeof(iso9660_dentry))) {
+            if ((i == 0) && (b_offs == 0)) {
                 if (dentry->fi_len != 0) {
                     // XXX
                 }
@@ -428,8 +458,17 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                 strncpy(in_node->inode.fn, a_fn,
                     ISO9660_MAXNAMLEN_STD + 1);
 
-                if (sizeof(iso9660_dentry) % 2)
-                    b_offs++;
+                /* for all directories except the root, we skip processing the "." and ".." entries because
+                 * they duplicate the other entires and the dent_walk code will rely on the offset 
+                 * for the entry in the parent directory. */
+                if (count != 0) {
+                    free(in_node);
+                    in_node = NULL;
+                    b_offs += dentry->entry_len;
+                    dentry = (iso9660_dentry *) & buf[b_offs];
+                    b_offs += dentry->entry_len;
+                    continue;
+                }
             }
             else {
                 char *file_ver;
@@ -440,7 +479,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     UTF8 *name8;
                     int retVal;
 
-                    name16 = (UTF16 *) & buf[b_offs];
+                    name16 =
+                        (UTF16 *) & buf[b_offs + sizeof(iso9660_dentry)];
                     // the name is in UTF-16 BE -- convert to LE if needed
                     if (fs->endian & TSK_LIT_ENDIAN) {
                         int a;
@@ -455,9 +495,10 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     retVal =
                         tsk_UTF16toUTF8(fs->endian,
                         (const UTF16 **) &name16,
-                        (UTF16 *) & buf[b_offs + dentry->fi_len], &name8,
-                        (UTF8 *) ((uintptr_t) & in_node->
-                            inode.fn[ISO9660_MAXNAMLEN_STD]),
+                        (UTF16 *) & buf[b_offs + sizeof(iso9660_dentry) +
+                            dentry->fi_len], &name8,
+                        (UTF8 *) ((uintptr_t) & in_node->inode.
+                            fn[ISO9660_MAXNAMLEN_STD]),
                         TSKlenientConversion);
                     if (retVal != TSKconversionOK) {
                         if (tsk_verbose)
@@ -476,7 +517,8 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     if (readlen > ISO9660_MAXNAMLEN_STD)
                         readlen = ISO9660_MAXNAMLEN_STD;
 
-                    memcpy(in_node->inode.fn, &buf[b_offs], readlen);
+                    memcpy(in_node->inode.fn,
+                        &buf[b_offs + sizeof(iso9660_dentry)], readlen);
                     in_node->inode.fn[readlen] = '\0';
                 }
                 else {
@@ -500,14 +542,9 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                     '.')
                     in_node->inode.fn[strlen(in_node->inode.fn) - 1] =
                         '\0';
-
-                /* skip past padding byte */
-                b_offs += dentry->fi_len;
-                if (!(dentry->fi_len % 2)) {
-                    b_offs++;
-                }
             }
 
+
             // copy the raw dentry data into the node
             memcpy(&(in_node->inode.dr), dentry, sizeof(iso9660_dentry));
 
@@ -515,6 +552,12 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
             in_node->offset =
                 tsk_getu32(fs->endian, dentry->ext_loc_m) * fs->block_size;
             in_node->ea_size = dentry->ext_len;
+            in_node->dentry_offset = s_offs + b_offs;
+
+            if (is_first)
+                in_node->inode.is_orphan = 0;
+            else
+                in_node->inode.is_orphan = 1;
 
             /* record size to make sure fifos show up as unique files */
             in_node->size =
@@ -527,77 +570,92 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
                 int extra_bytes =
                     dentry->entry_len - sizeof(iso9660_dentry) -
                     dentry->fi_len;
-                // this takes care of the length adjustment that we already did
-                // on offs
-                if (extra_bytes % 2)
-                    extra_bytes--;
 
                 in_node->inode.rr =
-                    parse_susp(fs, &buf[b_offs], extra_bytes, NULL);
-                in_node->inode.susp_off = b_offs + s_offs;
+                    parse_susp(fs,
+                    &buf[b_offs + sizeof(iso9660_dentry) + dentry->fi_len],
+                    extra_bytes, NULL);
+                in_node->inode.susp_off =
+                    b_offs + sizeof(iso9660_dentry) + dentry->fi_len +
+                    s_offs;
                 in_node->inode.susp_len = extra_bytes;
 
                 if (in_node->inode.rr == NULL) {
                     // return -1;
                     // @@@ Verbose error
                 }
-                b_offs += extra_bytes;
             }
             else {
                 in_node->inode.susp_off = 0;
                 in_node->inode.susp_len = 0;
             }
 
-
             /* add inode to the list */
-            /* list not empty */
             if (iso->in_list) {
-                iso9660_inode_node *tmp;
-                tmp = iso->in_list;
-                while ((tmp->next)
-                    && ((in_node->offset != tmp->offset)
-                        || (!in_node->size)
-                        || (!tmp->size)))
-                    tmp = tmp->next;
+                iso9660_inode_node *tmp, *prev_tmp;
 
-                /* see if the file is already in list */
-                if ((in_node->offset == tmp->offset) && (in_node->size)
-                    && (tmp->size)) {
-                    if ((in_node->inode.rr) && (!tmp->inode.rr)) {
-                        tmp->inode.rr = in_node->inode.rr;
-                        in_node->inode.rr = NULL;
+                for (tmp = iso->in_list; tmp; tmp = tmp->next) {
+                    /* When processing the "first" volume descriptor, all entries get added to the list.
+                     * for the later ones, we skip duplicate ones that overlap with entries from a 
+                     * previous volume descriptor. */
+                    if ((in_node->offset == tmp->offset)
+                        && (in_node->size == tmp->size)
+                        && (in_node->size) && (is_first == 0)) {
+                        if (in_node->inode.rr) {
+                            if (tmp->inode.rr == NULL) {
+                                tmp->inode.rr = in_node->inode.rr;
+                                tmp->inode.susp_off =
+                                    in_node->inode.susp_off;
+                                tmp->inode.susp_len =
+                                    in_node->inode.susp_len;
+                                in_node->inode.rr = NULL;
+                            }
+                            else {
+                                free(in_node->inode.rr);
+                            }
+                        }
+
+                        if (tsk_verbose)
+                            tsk_fprintf(stderr,
+                                "iso9660_load_inodes_dir: Removing duplicate entry for: %s\n",
+                                in_node->inode.fn);
+                        free(in_node);
+                        in_node = NULL;
+                        count--;
+                        break;
                     }
-                    if (in_node->inode.rr)
-                        free(in_node->inode.rr);
-
-                    free(in_node);
-                    count--;
-
+                    prev_tmp = tmp;
                 }
-                /* file wasn't in list, add it */
-                else {
-                    tmp->next = in_node;
+
+                // add it to the end (if we didn't get rid of it above)
+                if (in_node) {
+                    prev_tmp->next = in_node;
                     in_node->next = NULL;
                 }
-
-                /* list is empty */
             }
             else {
                 iso->in_list = in_node;
                 in_node->next = NULL;
             }
+
+            // skip two entries if this was the root directory (the . and ..).
+            if ((i == 0) && (b_offs == 0) && (count == 1)) {
+                b_offs += dentry->entry_len;
+                dentry = (iso9660_dentry *) & buf[b_offs];
+            }
+            b_offs += dentry->entry_len;
         }
         s_offs += cnt1;
     }
-
     return count;
 }
 
 
 /**
- * Process the path table for a joliet secondary volume descriptor.
- * This will load each
- * of the directories in the table pointed to by he SVD.
+ * Process the path table for a joliet secondary volume descriptor
+ * and load all of the files pointed to it.
+ * The path table contains an entry for each directory.  This code
+ * then locates each of the diretories and proceses the contents. 
  *
  * @param fs File system to process
  * @param svd Pointer to the secondary volume descriptor
@@ -606,7 +664,7 @@ iso9660_load_inodes_dir(TSK_FS_INFO * fs, TSK_OFF_T a_offs, int count,
  */
 static int
 iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
-    int count)
+    int count, uint8_t is_first)
 {
     TSK_OFF_T pt_offs;          /* offset of where we are in path table */
     size_t pt_len;              /* bytes left in path table */
@@ -629,7 +687,7 @@ iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
         UTF16 *name16;
         UTF8 *name8;
 
-        // read the next entry
+        // Read the path table entry
         cnt = tsk_fs_read(fs, pt_offs, (char *) &dir, (int) sizeof(dir));
         if (cnt != sizeof(dir)) {
             if (cnt >= 0) {
@@ -646,6 +704,7 @@ iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
         if (dir.len_di > ISO9660_MAXNAMLEN_JOL)
             readlen = ISO9660_MAXNAMLEN_JOL;
 
+        memset(utf16_buf, 0, ISO9660_MAXNAMLEN_JOL);
         /* get UCS-2 filename for the entry */
         cnt = tsk_fs_read(fs, pt_offs, (char *) utf16_buf, readlen);
         if (cnt != dir.len_di) {
@@ -695,9 +754,10 @@ iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
             (TSK_OFF_T) (tsk_getu32(fs->endian,
                 dir.ext_loc) * fs->block_size);
 
+        // process the directory contents
         count =
             iso9660_load_inodes_dir(fs, extent, count,
-            ISO9660_CTYPE_UTF16, utf8buf);
+            ISO9660_CTYPE_UTF16, utf8buf, is_first);
 
         if (count == -1) {
             return -1;
@@ -707,8 +767,8 @@ iso9660_load_inodes_pt_joliet(TSK_FS_INFO * fs, iso9660_svd * svd,
 }
 
 /**
- * Proces the path table and identify the directories that are listed.  The contents of each directory will also
- * be processed.  The result will be that the list of inodes in the image will be loaded in ISO_INFO.
+ * Proces the path table and the directories that are listed in it.  
+ * The files in each directory will be stored in ISO_INFO. 
  *
  * @param iso File system to analyze and store results in
  * @returns -1 on error or count of inodes found.
@@ -726,6 +786,7 @@ iso9660_load_inodes_pt(ISO_INFO * iso)
     size_t pt_len;              /* bytes left in path table */
     TSK_OFF_T extent;           /* offset of extent for current directory */
     ssize_t cnt;
+    uint8_t is_first = 1;
 
     if (tsk_verbose)
         tsk_fprintf(stderr, "iso9660_load_inodes_pt: iso: %lu\n",
@@ -744,10 +805,13 @@ iso9660_load_inodes_pt(ISO_INFO * iso)
         if ((s->svd.esc_seq[0] == 0x25) && (s->svd.esc_seq[1] == 0x2F) &&
             ((s->svd.esc_seq[2] == 0x40) || (s->svd.esc_seq[2] == 0x43)
                 || (s->svd.esc_seq[2] == 0x45))) {
-            count = iso9660_load_inodes_pt_joliet(fs, &(s->svd), count);
+            count =
+                iso9660_load_inodes_pt_joliet(fs, &(s->svd), count,
+                is_first);
             if (count == -1) {
                 return -1;
             }
+            is_first = 0;
         }
     }
 
@@ -763,7 +827,7 @@ iso9660_load_inodes_pt(ISO_INFO * iso)
         while (pt_len > 0) {
             int readlen;
 
-            /* get next dir... */
+            /* get next path table entry... */
             cnt = tsk_fs_read(fs, pt_offs, (char *) &dir, sizeof(dir));
             if (cnt != sizeof(dir)) {
                 if (cnt >= 0) {
@@ -805,9 +869,10 @@ iso9660_load_inodes_pt(ISO_INFO * iso)
                 (TSK_OFF_T) (tsk_getu32(fs->endian,
                     dir.ext_loc) * fs->block_size);
 
+            // process the directory contents
             count =
                 iso9660_load_inodes_dir(fs, extent, count,
-                ISO9660_CTYPE_ASCII, fn);
+                ISO9660_CTYPE_ASCII, fn, is_first);
 
             if (count == -1) {
                 return -1;
@@ -831,10 +896,6 @@ iso9660_dinode_load(ISO_INFO * iso, TSK_INUM_T inum)
 {
     iso9660_inode_node *n;
 
-    if (tsk_verbose)
-        tsk_fprintf(stderr, "iso9660_dinode_load: iso: %lu"
-            " inum: %" PRIuINUM "\n", (uintptr_t) iso, inum);
-
     n = iso->in_list;
     while (n && (n->inum != inum))
         n = n->next;
@@ -890,10 +951,6 @@ iso9660_dinode_copy(ISO_INFO * iso, TSK_FS_META * fs_meta)
     TSK_FS_INFO *fs = (TSK_FS_INFO *) & iso->fs_info;
     struct tm t;
 
-    if (tsk_verbose)
-        tsk_fprintf(stderr, "iso9660_dinode_copy: iso: %lu"
-            " inode: %lu\n", (uintptr_t) iso, (uintptr_t) fs_meta);
-
     if (fs_meta == NULL) {
         tsk_errno = TSK_ERR_FS_ARG;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
@@ -953,7 +1010,12 @@ iso9660_dinode_copy(ISO_INFO * iso, TSK_FS_META * fs_meta)
     ((TSK_DADDR_T *) fs_meta->content_ptr)[0] =
         (TSK_DADDR_T) tsk_getu32(fs->endian, iso->dinode->dr.ext_loc_m);
 
-    fs_meta->flags = TSK_FS_META_FLAG_ALLOC;
+    // mark files that were found from other volume descriptors as unalloc so that they 
+    // come up as orphan files. 
+    if (iso->dinode->is_orphan)
+        fs_meta->flags = TSK_FS_META_FLAG_UNALLOC | TSK_FS_META_FLAG_USED;
+    else
+        fs_meta->flags = TSK_FS_META_FLAG_ALLOC | TSK_FS_META_FLAG_USED;
     return 0;
 }
 
@@ -976,11 +1038,6 @@ iso9660_inode_lookup(TSK_FS_INFO * fs, TSK_FS_FILE * a_fs_file,
         return 1;
     }
 
-    // load the inode into the ISO buffer
-    if (iso9660_dinode_load(iso, inum)) {
-        return 1;
-    }
-
     if (a_fs_file->meta == NULL) {
         if ((a_fs_file->meta =
                 tsk_fs_meta_alloc(ISO9660_FILE_CONTENT_LEN)) == NULL)
@@ -990,9 +1047,23 @@ iso9660_inode_lookup(TSK_FS_INFO * fs, TSK_FS_FILE * a_fs_file,
         tsk_fs_meta_reset(a_fs_file->meta);
     }
 
-    // copy into the FS_META structure
-    if (iso9660_dinode_copy(iso, a_fs_file->meta)) {
-        return 1;
+    // see if they are looking for the special "orphans" directory
+    if (inum == TSK_FS_ORPHANDIR_INUM(fs)) {
+        if (tsk_fs_dir_make_orphan_dir_meta(fs, a_fs_file->meta))
+            return 1;
+        else
+            return 0;
+    }
+    else {
+        // load the inode into the ISO buffer
+        if (iso9660_dinode_load(iso, inum)) {
+            return 1;
+        }
+
+        // copy into the FS_META structure
+        if (iso9660_dinode_copy(iso, a_fs_file->meta)) {
+            return 1;
+        }
     }
 
     return 0;
@@ -1004,7 +1075,7 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
 {
     char *myname = "iso9660_inode_walk";
     ISO_INFO *iso = (ISO_INFO *) fs;
-    TSK_INUM_T inum;
+    TSK_INUM_T inum, end_inum_tmp;
     TSK_FS_FILE *fs_file;
     int myflags;
 
@@ -1038,8 +1109,12 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
         return 1;
     }
 
+    /* If ORPHAN is wanted, then make sure that the flags are correct */
     if (flags & TSK_FS_META_FLAG_ORPHAN) {
-        return 0;
+        flags |= TSK_FS_META_FLAG_UNALLOC;
+        flags &= ~TSK_FS_META_FLAG_ALLOC;
+        flags |= TSK_FS_META_FLAG_USED;
+        flags &= ~TSK_FS_META_FLAG_UNUSED;
     }
     else if (((flags & TSK_FS_META_FLAG_ALLOC) == 0) &&
         ((flags & TSK_FS_META_FLAG_UNALLOC) == 0)) {
@@ -1054,6 +1129,20 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
         flags |= (TSK_FS_META_FLAG_USED | TSK_FS_META_FLAG_UNUSED);
     }
 
+    /* If we are looking for orphan files and have not yet filled
+     * in the list of unalloc inodes that are pointed to, then fill
+     * in the list
+     * */
+    if ((flags & TSK_FS_META_FLAG_ORPHAN)
+        && (fs->list_inum_named == NULL)) {
+        if (tsk_fs_dir_load_inum_named(fs) != TSK_OK) {
+            strncat(tsk_errstr2,
+                " - iso9660_inode_walk: identifying inodes allocated by file names",
+                TSK_ERRSTR_L);
+            return 1;
+        }
+    }
+
 
     if ((fs_file = tsk_fs_file_alloc(fs)) == NULL)
         return 1;
@@ -1062,21 +1151,39 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
             tsk_fs_meta_alloc(ISO9660_FILE_CONTENT_LEN)) == NULL)
         return 1;
 
+    // we need to handle fs->last_inum specially because it is for the 
+    // virtual ORPHANS directory.  Handle it outside of the loop.
+    if (last == TSK_FS_ORPHANDIR_INUM(fs))
+        end_inum_tmp = last - 1;
+    else
+        end_inum_tmp = last;
+
     /*
      * Iterate.
      */
-    for (inum = start; inum <= last; inum++) {
+    for (inum = start; inum <= end_inum_tmp; inum++) {
         int retval;
         if (iso9660_dinode_load(iso, inum)) {
             tsk_fs_file_close(fs_file);
             return 1;
         }
 
+        if (iso9660_dinode_copy(iso, fs_file->meta))
+            return 1;
+
+        myflags = fs_file->meta->flags;
+
         if ((flags & myflags) != myflags)
             continue;
 
-        if (iso9660_dinode_copy(iso, fs_file->meta))
-            return 1;
+        /* If we want only orphans, then check if this
+         * inode is in the seen list
+         * */
+        if ((myflags & TSK_FS_META_FLAG_UNALLOC) &&
+            (flags & TSK_FS_META_FLAG_ORPHAN) &&
+            (tsk_list_find(fs->list_inum_named, inum))) {
+            continue;
+        }
 
         retval = action(fs_file, ptr);
         if (retval == TSK_WALK_ERROR) {
@@ -1088,6 +1195,29 @@ iso9660_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start, TSK_INUM_T last,
         }
     }
 
+    // handle the virtual orphans folder if they asked for it
+    if ((last == TSK_FS_ORPHANDIR_INUM(fs))
+        && (flags & TSK_FS_META_FLAG_ALLOC)
+        && (flags & TSK_FS_META_FLAG_USED)) {
+        int retval;
+
+        if (tsk_fs_dir_make_orphan_dir_meta(fs, fs_file->meta)) {
+            tsk_fs_file_close(fs_file);
+            return 1;
+        }
+        /* call action */
+        retval = action(fs_file, ptr);
+        if (retval == TSK_WALK_STOP) {
+            tsk_fs_file_close(fs_file);
+            return 0;
+        }
+        else if (retval == TSK_WALK_ERROR) {
+            tsk_fs_file_close(fs_file);
+            return 1;
+        }
+    }
+
+
     /*
      * Cleanup.
      */
@@ -1360,11 +1490,10 @@ iso9660_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 
     for (p = iso->pvd; p != NULL; p = p->next) {
         i++;
-        tsk_fprintf(hFile, "\nPRIMARY VOLUME DESCRIPTOR %d\n", i);
-        tsk_fprintf(hFile, "\nFILE SYSTEM INFORMATION\n");
+        tsk_fprintf(hFile, "\n=== PRIMARY VOLUME DESCRIPTOR %d ===\n", i);
+        tsk_fprintf(hFile, "FILE SYSTEM INFORMATION\n");
         tsk_fprintf(hFile,
             "--------------------------------------------\n");
-        tsk_fprintf(hFile, "Read from Primary Volume Descriptor\n");
         tsk_fprintf(hFile, "File System Type: ISO9660\n");
         tsk_fprintf(hFile, "Volume Name: %s\n", p->pvd.vol_id);
         tsk_fprintf(hFile, "Volume Set Size: %d\n",
@@ -1441,6 +1570,8 @@ iso9660_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 
         tsk_fprintf(hFile, "Inode Range: %" PRIuINUM " - %" PRIuINUM "\n",
             fs->first_inum, fs->last_inum);
+        tsk_fprintf(hFile, "Root Directory Block: %" PRIuDADDR "\n",
+            tsk_getu32(fs->endian, p->pvd.dir_rec.ext_loc_m));
 
         tsk_fprintf(hFile, "\nCONTENT INFORMATION\n");
         tsk_fprintf(hFile,
@@ -1461,11 +1592,11 @@ iso9660_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 
     for (s = iso->svd; s != NULL; s = s->next) {
         i++;
-        tsk_fprintf(hFile, "\nSUPPLEMENTARY VOLUME DESCRIPTOR %d\n", i);
-        tsk_fprintf(hFile, "\nFILE SYSTEM INFORMATION\n");
+        tsk_fprintf(hFile,
+            "\n=== SUPPLEMENTARY VOLUME DESCRIPTOR %d ===\n", i);
+        tsk_fprintf(hFile, "FILE SYSTEM INFORMATION\n");
         tsk_fprintf(hFile,
             "--------------------------------------------\n");
-        tsk_fprintf(hFile, "Read from Supplementary Volume Descriptor\n");
         tsk_fprintf(hFile, "File System Type: ISO9660\n");
         tsk_fprintf(hFile, "Volume Name: %s\n", s->svd.vol_id);
         tsk_fprintf(hFile, "Volume Set Size: %d\n",
@@ -1542,6 +1673,9 @@ iso9660_fsstat(TSK_FS_INFO * fs, FILE * hFile)
                 s->svd.pt_loc_m) + tsk_getu32(fs->endian,
                 s->svd.pt_size_m) / fs->block_size);
 
+        tsk_fprintf(hFile, "Root Directory Block: %" PRIuDADDR "\n",
+            tsk_getu32(fs->endian, s->svd.dir_rec.ext_loc_m));
+
         /* learn joliet level (1-3) */
         if (!strncmp((char *) s->svd.esc_seq, "%/E", 3))
             tsk_fprintf(hFile, "Joliet Name Encoding: UCS-2 Level 3\n");
@@ -2250,6 +2384,7 @@ iso9660_open(TSK_IMG_INFO * img_info, TSK_OFF_T offset,
         iso9660_close(fs);
         return NULL;
     }
+    fs->inum_count++;           // account for the orphan directory
 
     fs->last_inum = fs->inum_count - 1;
     fs->first_inum = ISO9660_FIRSTINO;
