@@ -19,9 +19,7 @@
 #if HAVE_LIBEWF
 
 #if defined(TSK_WIN32)
-#define HAVE_WIDE_CHARACTER_TYPE 1
-#define HAVE_WIDE_CHARACTER_SUPPORT_FUNCTIONS 1
-#include <config_windows.h>
+#include <config_msc.h>
 #endif
 
 #include <libewf.h>
@@ -30,7 +28,8 @@
 extern "C" {
 #endif
 
-    extern TSK_IMG_INFO *ewf_open(int, const TSK_TCHAR **);
+    extern TSK_IMG_INFO *ewf_open(int, const TSK_TCHAR * const images[],
+        unsigned int a_ssize);
 
     typedef struct {
         TSK_IMG_INFO img_info;
